import struct

civ           = 138
eSTATE_P1     = 0
eSTATE_P2     = 1 
eSTATE_TOADDR = 2
eSTATE_FMADDR = 3
eSTATE_CMD    = 4
eSTATE_EOM    = 5
state         = eSTATE_P1
data_mode_    = True
rxcmd         = 0
name          = "Icom F8101 (python)"

volume_table = [[0x00, 0x00],\
                [0x01, 0x39],\
                [0x01, 0x43],\
                [0x01, 0x47],\
                [0x01, 0x51],\
                [0x01, 0x55],\
                [0x01, 0x59],\
                [0x01, 0x63],\
                [0x01, 0x67],\
                [0x01, 0x71],\
                [0x01, 0x75],\
                [0x01, 0x77],\
                [0x01, 0x79],\
                [0x01, 0x81],\
                [0x01, 0x83],\
                [0x01, 0x85],\
                [0x01, 0x87],\
                [0x01, 0x89],\
                [0x01, 0x91],\
                [0x01, 0x93],\
                [0x01, 0x95],\
                [0x01, 0x97],\
                [0x01, 0x99],\
                [0x02, 0x01],\
                [0x02, 0x03],\
                [0x02, 0x05],\
                [0x02, 0x07],\
                [0x02, 0x09],\
                [0x02, 0x11],\
                [0x02, 0x13],\
                [0x02, 0x15],\
                [0x02, 0x17],\
                [0x02, 0x19],\
                [0x02, 0x21],\
                [0x02, 0x23],\
                [0x02, 0x25],\
                [0x02, 0x27],\
                [0x02, 0x29],\
                [0x02, 0x31],\
                [0x02, 0x33],\
                [0x02, 0x35],\
                [0x02, 0x37],\
                [0x02, 0x39],\
                [0x02, 0x41],\
                [0x02, 0x43],\
                [0x02, 0x45],\
                [0x02, 0x47],\
                [0x02, 0x49],\
                [0x02, 0x51],\
                [0x02, 0x53],\
                [0x02, 0x55]]

def getName():
    return bytes(name, 'utf-8')

def getCapModes():
    return bytes("USB,LSB,USB,LSB,AM", 'utf-8')

def getCapAgcLevels():
    return bytes("AGC-F,AGC-S,AGC-A,AGC-O", 'utf-8')

def cmdVolume(level):
    msg = []
    msg.append(0xfe)  # preamble
    msg.append(0xfe)  # preamble
    msg.append(civ)   # radio address
    msg.append(0xe0)  # controller address
    msg.append(0x14)  # cmd
    msg.append(0x01)  # subcmd (send memory channel)
    
    volidx = int(51.0 * level)
    msb = volume_table[volidx][0]
    lsb = volume_table[volidx][1]
    
    msg.append(msb)  # MSB
    msg.append(lsb)  # LSB
    
    msg.append(0xFD) # footer
    bytes = struct.pack("{}B".format(len(msg)), *msg)
    
    return (True, bytes)

def cmdAgc(agc):
    msg = []
    msg.append(0xfe) # preamble
    msg.append(0xfe) # preamble
    msg.append(civ)  # radio address
    msg.append(0xe0) # controller address
    msg.append(0x1a) # cmd
    msg.append(0x05) # subcmd (send memory channel)
    msg.append(0x03) # category
    msg.append(0x06) # Item
    msg.append(0x00) # MSB

    if agc == "AGC-O":
        msg.append(0x00)
    elif agc == "AGC-F":
        msg.append(0x01)
    elif agc == "AGC-S":
        msg.append(0x02)
    elif agc == "AGC-A":
        msg.append(0x03)
    else:
        msg.append(0x01);

    msg.append(0xfd) # footer
    bytes = struct.pack("{}B".format(len(msg)), *msg)

    return (True, bytes)

def cmdTxFrequency(frequency):
    # just a nop (using same freq for both)
    return (False, b'')

def cmdRxFrequency(frequency):
    
    if frequency > 30000000 or frequency < 30000:
        # do nothing
        return (False, b'')

    e = int((int((frequency % 100) / 10) << 4) | (frequency % 10))
    d = int((int((frequency % 10000) / 1000) << 4) | int((frequency % 1000)/100))
    c = int((int((frequency % 1000000) / 100000) << 4) | int((frequency % 100000)/10000))
    b = int((int((frequency % 100000000) / 10000000) << 4) | int((frequency % 10000000)/1000000))
    a = 0

    msg = []

    msg.append(0xFE) # preamble
    msg.append(0xFE) # preamble
    msg.append(civ)  # radio address
    msg.append(0xE0) # controller address
    msg.append(0x1A) # cmd
    msg.append(0x35) # subcmd
    msg.append(e)    # 10/1             Hz
    msg.append(d)    # 1000/100         Hz
    msg.append(c)    # 10000/100000     Hz
    msg.append(b)    # 1000000/10000000 Hz
    msg.append(a)    # 0/0
    msg.append(0xFD) # footer

    bytes = struct.pack("{}B".format(len(msg)), *msg)
    
    return (True, bytes)

def cmdVfoModeA():
    # enter VFO mode (A)
    msg = []
    msg.append(0xFE) # preamble
    msg.append(0xFE) # preamble
    msg.append(civ)  # radio address
    msg.append(0xE0) # controller address
    msg.append(0x07) # cmd
    msg.append(0x00) # subcmd (VFO A mode)
    msg.append(0xFD) # footer

    bytes = struct.pack("{}B".format(len(msg)), *msg)

    return (True, bytes)

def cmdVfoModeB():
    # enter VFO mode (B)
    msg = []
    msg.append(0xFE) # preamble
    msg.append(0xFE) # preamble
    msg.append(civ)  # radio address
    msg.append(0xE0) # controller address
    msg.append(0x07) # cmd
    msg.append(0x01) # subcmd (VFO B mode)
    msg.append(0xFD) # footer

    bytes = struct.pack("{}B".format(len(msg)), *msg)

    return (True, bytes)

def cmdVfoSplit(enable):

    val = 0x01 if enable else 0x00
    
    # enter VFO mode
    msg = []
    msg.append(0xFE) # preamble
    msg.append(0xFE) # preamble
    msg.append(civ)  # radio address
    msg.append(0xE0) # controller address
    msg.append(0x1A) # cmd
    msg.append(0x05) #
    msg.append(0x03) #
    msg.append(0x17) # VFO split
    msg.append(0x00) # MSB
    msg.append(val)  # LSB
    msg.append(0xFD) # footer
    bytes = struct.pack("{}B".format(len(msg)), *msg)
    
    return (True, bytes)

def cmdMode(mode):
    global data_mode_
    modeval = 0x0
    
    if data_mode_:
        if mode == b"LSB":
            modeval = 0x0018
        elif mode == b"USB":
            modeval = 0x0019
        elif mode == b"UNK":
            modeval = 0x0019 # default to USB-D1
        else:
            print("MODE %s NOT SUPPORTED" % mode)
            return (False, b'')
    else:
        if mode == b"LSB":
            modeval = 0x0000
        elif mode == b"USB":
            modeval = 0x0001
        elif mode == b"AM":
            modeval = 0x0002
        elif mode == b"CW":
            modeval = 0x0003
        elif mode == b"RTTY":
            modeval = 0x0004
        else:
            print("MODE %s NOT SUPPORTED" % mode)
            return (False, b'')

    msg = []
    msg.append(0xFE) # preamble
    msg.append(0xFE) # preamble
    msg.append(civ)  # radio address
    msg.append(0xE0) # controller address
    msg.append(0x1A) # cmd
    msg.append(0x36) # subcmd
    msg.append(((modeval >> 8)&0xff)) # value MSB
    msg.append((modeval & 0xff)) # value LSB
    msg.append(0xFD) # footer
    bytes = struct.pack("{}B".format(len(msg)), *msg)
    
    return (True, bytes)

def cmdPtt(val):
    msg = []
    msg.append(0xFE) # preamble
    msg.append(0xFE) # preamble
    msg.append(civ)  # radio address
    msg.append(0xE0) # controller address
    msg.append(0x1A) # cmd
    msg.append(0x37) # subcmd (send memory channel)
    msg.append(0x00) # MSB

    if val:
        msg.append(0x02)
    else:
        msg.append(0x00)

    msg.append(0xFD) # footer
    bytes = struct.pack("{}B".format(len(msg)), *msg)
        
    return (True, bytes)

def setCivAddress(newciv):
    global civ
    if newciv < 255:
        civ = newciv

def setDataMode():
    global data_mode_
    data_mode_ = True

def setVoiceMode():
    global data_mode_
    data_mode_ = False
        
def onRxData(msg):
    global state
    global rxcmd
    complete = False

    bytes = struct.unpack('{}B'.format(len(msg)),msg)

    for byte in bytes:
        if state == eSTATE_P1:
            if byte == 0xFE:
                state = eSTATE_P2
        elif state == eSTATE_P2:
            if byte == 0xFE:
                state = eSTATE_TOADDR
            else:
                print("unexpected byte waiting for P2: %d" % byte)
                state = eSTATE_P1 # junk, ignore
        elif state == eSTATE_TOADDR:
            if byte == 0xE0:
                state = eSTATE_FMADDR
            else:
                state = eSTATE_P1 # not for us, ignore
        elif state == eSTATE_FMADDR:
            if byte == civ:
                state = eSTATE_CMD
            else:
                print("unexpected byte waiting for FMADDR: %d" % byte)
                state = eSTATE_P1 # not for us, ignore
        elif state == eSTATE_CMD: # only expect single cmd (ack/nak), else
            # rest will be dumped or we timeout
            state = eSTATE_EOM
            rxcmd = byte
        elif state == eSTATE_EOM:
            if byte == 0xFD:
                complete = True
            else:
                print("unexpected byte waiting for EOM: " % byte)

            # always go back to P1
            state = eSTATE_P1

        # now handle any received ACKs/NAKs
        if complete:
            if rxcmd == 0xFB:
                return 1
            elif rxcmd == 0xFA:
                print("CMD_REJECTED serial errors detected")
                return -1
            else:
                print("cmd not handled: %d" % rxcmd);
                return -1
        
    return 0
